@@ -1,9 +1,9 @@
 """
 Module containing all the load/dump code for all file formats
 """
-from typing import IO, Any, Callable, Dict
 
-from path import Path
+from pathlib import Path
+from typing import IO, Any, Callable, Dict
 
 from jubeatools.song import Song
 
@@ -26,11 +26,9 @@ from .memon import (
     load_memon_0_2_0,
     load_memon_legacy,
 )
-from .typing import Dumper
+from .typing import Dumper, Loader
 
-# Loaders deserialize a Path to a Song object
-# The Path can be a file or a folder depending on the format
-LOADERS: Dict[Format, Callable[[Path], Song]] = {
+LOADERS: Dict[Format, Loader] = {
     Format.MEMON_LEGACY: load_memon_legacy,
     Format.MEMON_0_1_0: load_memon_0_1_0,
     Format.MEMON_0_2_0: load_memon_0_2_0,
@@ -40,7 +38,7 @@ LOADERS: Dict[Format, Callable[[Path], Song]] = {
     Format.MEMO_2: load_memo2,
 }
 
-DUMPERS: Dict[str, Dumper] = {
+DUMPERS: Dict[Format, Dumper] = {
     Format.MEMON_LEGACY: dump_memon_legacy,
     Format.MEMON_0_1_0: dump_memon_0_1_0,
     Format.MEMON_0_2_0: dump_memon_0_2_0,
